package gelwood.crossword;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.util.Random;

public class Dictionary {
	
	private ArrayList<String> words;
	
	private int length;
	
	private ArrayList<String> usedWords;
	
	private ArrayList<Integer> wordLengths;
	
	public Dictionary(String wordList){
		this.length = 0;
		this.words = new ArrayList<String>();
		
		this.usedWords = new ArrayList<String>();
		
		this.wordLengths = new ArrayList<Integer>();
		
		StringTokenizer myST = new StringTokenizer(wordList);
		
		while (myST.hasMoreTokens()) {
			String myToken = myST.nextToken();
			
			int myLength = myToken.length();
			if (!(this.wordLengths.contains(myLength))){
				this.wordLengths.add(myLength);
			}
			
			this.words.add(myToken);
		}
	}
	
	private Dictionary(ArrayList<String> words, int length) {
		this.words = words;
		this.length = length;
		this.usedWords = new ArrayList<String>();
	}
	
	protected Dictionary(Dictionary myDictionary){
		this.words = new ArrayList<String>(myDictionary.words);
		this.length = new Integer(myDictionary.length);
		this.usedWords = new ArrayList<String>();
	}
	
	protected ArrayList<Integer> getWordLengths(){
		return this.wordLengths;
	}
	
	public int getSize(){
		return this.words.size();
	}
	
	public int getLength(){
		return this.length;
	}
	
	protected void reset(){
		
		this.words.addAll(usedWords);
		this.usedWords = new ArrayList<String>();
	}
	
	protected ArrayList<Dictionary> getSubDictionaries(ArrayList<Integer> myLengths){
		Iterator<Integer> myIT = myLengths.iterator();
			
		ArrayList<Dictionary> subDictionaries = new ArrayList<Dictionary>();
			
		while (myIT.hasNext()){
			int myLength = myIT.next();
			Dictionary newDictionary = this.makeSubDictionary(myLength);
				
			subDictionaries.add(newDictionary);
		}	
		
		return subDictionaries;
	}
	
	protected boolean hasMore(){
		int size = this.words.size();
		
		if (size > 0){
			return true;
		} else {
			return false;
		}
	}
	
	
	protected String removeRandomWord(){
		Random myRandom = new Random();
		
		int size = this.words.size();
		
		if (size > 0){
			String myWord = this.words.remove(myRandom.nextInt(size));
		
			this.usedWords.add(myWord);
			return myWord;
			
		} else {
			throw new RuntimeException("Dictionary is empty");
		}
		
	}
	
	protected void removeMatching(int index, char myChar){
		ArrayList<String> removeList = new ArrayList<String>();
		
		Iterator<String> wordIt = this.words.iterator();
		
		while (wordIt.hasNext()) {
			String currentWord = wordIt.next();
			if (currentWord.charAt(index) != myChar) {
				//System.out.println(currentWord);
				removeList.add(currentWord);
			}
		}
		
		this.words.removeAll(removeList);
		this.usedWords.addAll(removeList);
	}
	
	protected ArrayList<String> getWords(){
		return this.words;
	}
	
	public Dictionary makeSubDictionary(int length){
		
		ArrayList<String> subDictWords = new ArrayList<String>();
		
		Iterator<String> myIt = this.words.iterator();
		
		while (myIt.hasNext()){
			String myString = myIt.next();
			
			if(myString.length() == length) {
				subDictWords.add(myString);
			}
			
		}
		
		return new Dictionary(subDictWords, length);
		
	}
	
	public void print() {
		Iterator<String> myIT = this.words.iterator();
		while(myIT.hasNext()) {
			String next = myIT.next();
			System.out.println(next);
		}
	}
}

