package gelwood.crossword;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.StringTokenizer;



public class Grid {

	
	private ArrayList<Square> grid;
	private ArrayList<Word> words;
	
	private ArrayList<Word> usedSlots;
	
	private String gridString;
	
	private int width;
	private int height;
	
	public Grid(String gridString){
		this.words = new ArrayList<Word>();
		this.gridString = gridString;
		
		this.usedSlots = new ArrayList<Word>();
		
		StringTokenizer myST = new StringTokenizer(this.gridString);
		String myString = myST.nextToken();
		
		this.width = myString.length();
		this.height = 1;
		
		while (myST.hasMoreTokens()){
			this.height = this.height + 1;
			myST.nextToken();
		}
		
		this.grid = this.makeEmptyGrid(gridString);
		
		this.numberGrid();
		
		this.weighIntersections();

		
	}

	protected boolean isUsed(String word){
		Iterator<Word> usedWords = this.usedSlots.iterator();
		
		return isUsed(word, usedWords);
	}
	
	private boolean isUsed(String word, Iterator<Word> usedWords) {
		if (usedWords.hasNext()) {
			Word myNext = usedWords.next();
			String myString = myNext.lettersAsString();
			
			if (myString.compareTo(word) == 0) {
				return true;
			} else {
				return isUsed(word, usedWords);
			}
		} else {
			return false;
		}
	}
	
	protected void removeSlot(Word slot){
		this.words.remove(slot);
		this.usedSlots.add(slot);
	}
	
	protected boolean isContradiction(){
		Iterator<Word> myWords = this.words.iterator();
		
		return this.isConHelper(myWords);
	}
	
	private boolean isConHelper(Iterator<Word> myWords) {
		if (myWords.hasNext()){
			Word myWord = myWords.next();
			myWord.updateDictionary();
			Dictionary myDictionary = myWord.subDictionary;
			boolean result = false;
			
			if (myDictionary.getSize() == 0 ){
				result = true;
			}
			
			return (result || this.isConHelper(myWords));
		} else {
			return false;
		}
	}
	
	
	private boolean intersect(Word word1, Word word2){
		if (word1 == word2) {
			return false;
		}
		
		
		boolean result = false;
		
		ArrayList<BlankSquare> squares1 = word1.letters;
		ArrayList<BlankSquare> squares2 = word2.letters;
		
		int i = 0;
		
		while (i < squares1.size()) {
			
			int j = 0;
			while (j < squares2.size() ) {
				
				if (squares1.get(i) == squares2.get(j)) {
					result = true;
				}
				j = j + 1;	
			}
			
			
			i = i + 1;
		}
		
		return result;
	}
	
	protected void addSlot(Word mySlot) {
		this.words.add(mySlot);
		this.usedSlots.remove(mySlot);
	}
	
	protected ArrayList<Word> getIntersectingWords(Word currentWord) {
		ArrayList<Word> intersections = new ArrayList<Word>();
		
		Iterator<Word> theseWords = this.words.iterator();
		
		
		while (theseWords.hasNext()){
			Word myWord = theseWords.next();
			
			if (intersect(myWord, currentWord)) {
				System.out.println("Adding");
				intersections.add(myWord);
			}
			
			myWord.print();
		}
		return intersections;
	}
	

	
	
	protected boolean hasMoreSlots(){
		return (this.words.size() > 0);
	}
	
	protected Word getFirstSlot(){
		return this.words.get(0);
	}
	
	protected ArrayList<Word> getSortedWordSlots(){
		this.sortWordsByWeight();
		return this.words;
	}
	
	protected void sortWordsByWeight(){
		Comparator<Word> myComparator = new WordWeightCompare();
		
		Collections.sort(this.words, myComparator);
	}
	
	protected void setSlot(Decision myDecision) {
		Word slot = myDecision.getSlot();
		String word = myDecision.getWord();
		
		slot.setWord(word);
		words.remove(slot);
		usedSlots.add(slot);
	}
	
	protected void resetSlot(Decision myDecision) {
		Word slot = myDecision.getSlot();
		String word = myDecision.getOld();
		
		slot.setWord(word);
		words.add(slot);
		usedSlots.remove(slot);
	}
	
	private void weighIntersections(){
		this.findIntersections();
		
		Iterator<Word> myWords = this.words.iterator();
		
		while (myWords.hasNext()) {
			Word myWord = myWords.next();
			if (myWord == null) {
				throw new RuntimeException("GOT NULL WORD");
			}
			myWord.weigh();
		}
	}
	
	private void findIntersections(){
		ArrayList<Square> visitedSquares = new ArrayList<Square>();
		
		Iterator<Word> myWords = this.words.iterator();
		

		while (myWords.hasNext()) {
			Word myWord = myWords.next();
			
			if (myWord != null) {
			
			Iterator<BlankSquare> mySquares = myWord.letters.iterator();
			
				while (mySquares.hasNext()) {
					BlankSquare mySquare = mySquares.next();
					
					if (visitedSquares.contains(mySquare)) {
						mySquare.makeIntersection();
					} else {
						visitedSquares.add(mySquare);
					}
				}
			}
		}
	}
	
	protected void setupSubDictionaries(Dictionary myDictionary) {
		ArrayList<Integer> gridLengths = this.getWordLengths();
		
		ArrayList<Dictionary> dictionaries = myDictionary.getSubDictionaries(gridLengths);
		
		
		WordLengthCompare wlcompare = new WordLengthCompare();
		
		Collections.sort(this.words, wlcompare);
		
		Iterator<Word> wordsIt = this.words.iterator();
		
		Dictionary currentD = new Dictionary("");
		
		while(wordsIt.hasNext()){
			
			Word myWord = wordsIt.next();
			int wordLength = myWord.getLength();

			
			if (wordLength != currentD.getLength()){
				currentD = getMatchingSub(wordLength, dictionaries);
			} 
			
			myWord.setSubD(new Dictionary(currentD));
			
			
		
		}
	}
	
	private Dictionary getMatchingSub(int length, ArrayList<Dictionary> dictionaries) {
		Iterator<Dictionary> dictsIt = dictionaries.iterator();
		
		
		
		
		while (dictsIt.hasNext()) {
			Dictionary currentD = dictsIt.next();
			if (currentD.getLength() == length){
				return currentD;
			}
		}
		
		//No words in Dictionary are the right length for this BlankWord
		return new Dictionary("");
	}

	private void numberGrid(){
		
		AcrossIterator myAIt = new AcrossIterator(this);
		this.findWords(myAIt);
		
		DownIterator myDIt = new DownIterator(this);
		this.findWords(myDIt);
		
		NumberIterator myNIt = new NumberIterator(this);
		
		while (myNIt.hasNext()) {
			myNIt.next();
		}
	}
	
	private void addWord(Word word){

		if (word != null){
			ArrayList<BlankSquare> letters = word.letters;
			if (letters.size() > 1) {
				System.out.println("word.print()");
				word.print();
				this.words.add(word);
			}
		}
	}
	
	private void findWords(Iterator<Square> myIterator) {
		Word word = null;
		
		while (myIterator.hasNext()) {
			Square mySquare = myIterator.next();
			
			if (mySquare instanceof BlankSquare) {
				BlankSquare myBlank = (BlankSquare)mySquare;
				
				if (
					myBlank.isRowStart() &&
					(myIterator instanceof AcrossIterator) )
				{
					addWord(word);
					word = new AcrossWord(myBlank);
				} else if (
					myBlank.isColumnStart() &&
					(myIterator instanceof DownIterator) )
				{
					addWord(word);
					word = new AcrossWord(myBlank);
				} else if ( 
					myBlank.isAcrossStart() && 
					(myIterator instanceof AcrossIterator) )
				{
					addWord(word);
					word = new AcrossWord(myBlank);
				} else if ( 
					myBlank.isDownStart() &&
					(myIterator instanceof DownIterator) )
				{
					addWord(word);
					word = new DownWord(myBlank);
				} else if (word != null) {
						word.addLetter(myBlank);
					}
				
			} else {
				addWord(word);
				word = null;
			}
		}
		addWord(word);

		System.out.println();
	}
	
	private ArrayList<Square> makeEmptyGrid(String gridString){
		
		StringTokenizer myST = new StringTokenizer(gridString);
		ArrayList<Square> mySquares = new ArrayList<Square>();
		
		while (myST.hasMoreTokens()) {
			String myString = myST.nextToken();
			char myChars[] = myString.toCharArray();
			
			int i = 0;
			
			while (i < myChars.length){
				Square mySquare = null;
				char myChar = myChars[i];
				
				if (myChar == '.') {
					mySquare = new BlankSquare();
				} else if (myChar == '#') {
					mySquare = new BlackSquare();
				} else {
					throw new RuntimeException("Badly formed grid string");
				}
				
				mySquares.add(mySquare);
				i = i + 1;
				
			}
		}
		
		return mySquares;	
	}
	
	protected ArrayList<Integer> getWordLengths(){
		ArrayList<Integer> myLengths = new ArrayList<Integer>();
		
		int i = 0;
		
		while (i < this.words.size()) {
			Word myWord = this.words.get(i);
			int length = myWord.getLength();
			
			if ( ! (myLengths.contains(length)) ){
				myLengths.add(length);
			}
			
			i = i + 1;
		}
		
		return myLengths;
	}
	
	
	public void print(){
		int myWidth = 1;
		int count = 0;
		int mySize = this.height * this.width;
		
		while (count < mySize) {
			Square mySquare = grid.get(count);
			mySquare.print();
			
			myWidth = myWidth + 1;
			
			if (myWidth > this.width){
				System.out.println("");
				myWidth = 1;
			}
		
			count = count + 1;
		}	
		
		
		System.out.println("\nwords:\n");
		
		
		Iterator<Word> it1 = this.words.iterator();
		while(it1.hasNext()){
			Word nextWord = it1.next();
			nextWord.print();
		}
		
		System.out.println("\nused slots\n");
		
		Iterator<Word> it2 = this.usedSlots.iterator();
		while(it2.hasNext()) {
			Word nextWord = it2.next();
			nextWord.print();
		}

		
	}
	
	public void printIntersection(){
		int myWidth = 1;
		int count = 0;
		int mySize = this.height * this.width;
		
		while (count < mySize) {
			Square mySquare = grid.get(count);
			
			if (mySquare.isIntersection()){
				System.out.print("I");
			} else {
				mySquare.print();
			}
			
			myWidth = myWidth + 1;
			
			if (myWidth > this.width){
				System.out.println("");
				myWidth = 1;
			}
		
			count = count + 1;
		}	
	}
	
	public void printRowStarts(){
		int myWidth = 1;
		int count = 0;
		int mySize = this.height * this.width;
		
		while (count < mySize) {
			Square mySquare = grid.get(count);
			
			if (mySquare.rowStart){
				System.out.print("R");
			} else {
				mySquare.print();
			}
			
			myWidth = myWidth + 1;
			
			if (myWidth > this.width){
				System.out.println("");
				myWidth = 1;
			}
		
			count = count + 1;
		}	
	}
	
	public void printColumnStarts(){
		int myWidth = 1;
		int count = 0;
		int mySize = this.height * this.width;
		
		while (count < mySize) {
			Square mySquare = grid.get(count);
			
			if (mySquare.rowStart){
				System.out.print("");
			} else {
				mySquare.print();
			}
			
			myWidth = myWidth + 1;
			
			if (myWidth > this.width){
				System.out.println("");
				myWidth = 1;
			}
		
			count = count + 1;
		}	
	}
	
	protected int numberOfSlots(){
		return this.words.size();
	}
	
	protected void updateSubDictionaries(){
		Iterator<Word> myWords = this.words.iterator();
		
		while(myWords.hasNext()) {
			Word myWord = myWords.next();
			
			myWord.updateDictionary();
		}
	}
	
	protected String chooseBestWord(Dictionary dictionary) {
		System.out.println("WRITE THIS");
		return "";
	}
	
	protected int size() {
		return this.grid.size();
	}
	
	protected Square getSquare(int i){
		return this.grid.get(i);
	}
	
	public int getWidth(){
		return this.width;
	}
	
	public int getHeight() {
		return this.height;
	}
	
	protected ArrayList<Square> getGrid(){
		return this.grid;
	}
	
	public void printWords() {
		int i = 0;
		while (i < this.words.size()) {
			Word myWord = this.words.get(i);
			myWord.print();
			i = i + 1;
		}
	}
}

