package gelwood.crossword;

import java.util.ArrayList;
import java.util.Iterator;

public abstract class Word {
	protected ArrayList<BlankSquare> letters;
	protected int number;
	protected Dictionary subDictionary;
	private int weight;
	
	public Word(BlankSquare blank){
		this.letters = new ArrayList<BlankSquare>();
		this.letters.add(blank);
		this.number = blank.getNumber();
		this.subDictionary = new Dictionary("");
		this.weight = 0;
	}
	
	protected Word(){
		this.letters = new ArrayList<BlankSquare>();
		this.number = 0;
		this.subDictionary = new Dictionary("");
		this.weight = 0;
	}
	
	protected void weigh(){
		Iterator<BlankSquare> mySquares = this.letters.iterator();
		
		while (mySquares.hasNext()) {
			BlankSquare mySquare = mySquares.next();
			
			if (mySquare.isIntersection()) {
				this.weight = this.weight + 1;
			}
		}
	}
	
	protected boolean contains(BlankSquare letter){
		return letters.contains(letter);
	}
	
	protected String lettersAsString(){
		Iterator<BlankSquare> myIterator = this.letters.iterator();
		
		return this.lettersAsStringHelp(myIterator);
	}
	
	private String lettersAsStringHelp(Iterator<BlankSquare> myIterator){
		if (myIterator.hasNext()) {
			BlankSquare mySquare = myIterator.next();
			char myChar = mySquare.getLeter();
			return myChar + this.lettersAsStringHelp(myIterator);
		} else {
			return "";
		}
	}
	
	protected void reset(){
		this.subDictionary.reset();
		this.updateDictionary();
	}
	
	protected void setWord(String myString) {
		
		int i = 0;
		
		while (i < myString.length()){
			BlankSquare letter = letters.get(i);
			
			letter.setLetter(myString.charAt(i));
			
			i = i + 1;
		}
			
	}
	
	protected int getHeuristic(){
		this.updateDictionary();
		return this.subDictionary.getSize();
	}
	
	protected void updateDictionary(){
		Iterator<BlankSquare> myIterator = this.letters.iterator();
		
		int index = 0;
		
		while (myIterator.hasNext()) {
			BlankSquare mySquare = myIterator.next();
			char myChar = mySquare.getLeter();
			if (mySquare.getLeter() != '.') {
				this.subDictionary.removeMatching(index, myChar);
			}
			index = index + 1;
		}
		
	}
	
	public int getWeight(){
		return this.weight;
	}
	
	public void addLetter(BlankSquare blank){
		this.letters.add(blank);
	}
	
	public ArrayList<BlankSquare> getLetters(){
		return this.letters;
	}
	
	public Dictionary getSubD(){
		return this.subDictionary;
	}
	
	public void setSubD(Dictionary newDict){
		Dictionary myDictionary = new Dictionary(newDict);
		this.subDictionary = myDictionary;
	}
	
	public int getNumber(){
		if (this.number == 0) {
			BlankSquare firstLetter = this.letters.get(0);
			
			this.number = firstLetter.getNumber();
		}
		
		return this.number;
	}
	
	public int getLength(){
		return this.letters.size();
	}
	
	public void print() {
		int i = 0;
		System.out.print("WORD :");
		while (i < letters.size()) {
			BlankSquare myLetter = letters.get(i);
			myLetter.print();
			i = i + 1;
		}
		
		System.out.print(":Dictionary is size: " + this.subDictionary.getSize());
		System.out.print(":\n");
		
		
	}
}

